#!/usr/bin/perl -w

# Copyright 2013, 2014, 2020 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


# Print a tikz picture which is the UArD tree row lines with the horizontals
# and diagonals separate.

use strict;
use List::Util 'max';

use FindBin;
use lib $FindBin::Bin;
use MyPrintwrap ':all';
use MyTriples 'pq_acceptable';

my $x_limit = 30;
my $y_limit = 15;

my $type = 'horizontals';
if (@ARGV) {
  $type = shift @ARGV;
}

# $m is a Math::Matrix.
# Multiply $m * [[$x],[$y]] column vector and return a list of the resulting
# pair ($xmul,$ymul).
sub matrix_mult_xy {
  my ($m, $x,$y) = @_;
  return mvec_to_pq($m * Math::Matrix->new([$x],[$y]));
};

if ($type eq 'horizontals') {
  print "  % generated by tools/$FindBin::Script $type\n";
  $Printwrap = 2;
  printwrap("\\draw");
  foreach my $r (1 .. $y_limit) {
    my $p = $r+1;
    my $q = $r;
    die unless pq_acceptable($p,$q);

    my ($x1,$y1) = matrix_mult_xy(U_2x2(), $p,$q);  # U*[p;q]
    my ($x2,$y2) = matrix_mult_xy(A_2x2(), $p,$q);  # A*[p;q]
    last if $y1 > $y_limit;

    $Printwrap = 4;
    printwrap_newline();
    for (;;) {
      if ($x1 >= $x_limit) { last; }           # stop at limit
      my $clipped = 0;
      if ($x2 >= $x_limit) { # clip end
        $clipped = 1;
        $x2 = $x_limit;
      }

      $x1 <  $x_limit or die;
      $x2 <= $x_limit or die;
      $y1 <= $y_limit or die;
      $y1 == $y2 or die;
      printwrap(" ($x1,$y1)--($x2,$y2)");
      $Printwrap = 6;
      last if $clipped;

      # D * [x1;y1]
      ($x1,$y1) = matrix_mult_xy(D_2x2(), $x1,$y1);
      ($x2,$y2) = matrix_mult_xy(D_2x2(), $x2,$y2);
    }
  }
  printwrap(";\n");
  print "  % end generated\n";

} elsif ($type eq 'diagonals') {

  print "    % generated by tools/$FindBin::Script $type\n";
  $x_limit = 29;
  $y_limit = 29;

  $Printwrap = 4;
  printwrap("\\draw");
  foreach my $d (0 .. max($x_limit,$y_limit)) {
    my $p = $d+1;    # p-q = d
    my $q = 1;
    next unless pq_acceptable($p,$q);

    my ($x1,$y1) = matrix_mult_xy(D_2x2(), $p,$q);  # D*[p;q]
    my ($x2,$y2) = matrix_mult_xy(A_2x2(), $p,$q);  # A*[p;q]
    last if $x1 > $x_limit;
    last if $y1 > $y_limit;

    $Printwrap = 6;
    printwrap_newline();

    for (;;) {
      die unless $x1 < $x2;
      die unless $y1 < $y2;
      die unless $x2 - $x1 == $d;
      die unless $y2 - $y1 == $d;
      if ($x1 >= $x_limit) { last; }           # stop at limit
      if ($y1 >= $y_limit) { last; }           # stop at limit

      my $clipped = 0;
      if ($x2 > $x_limit) {       # clip
        my $d = $x2 - $x_limit;
        $x2 -= $d;
        $y2 -= $d;
        $clipped = 1;
      }
      if ($y2 > $y_limit) {       # clip
        my $d = $y2 - $y_limit;
        $x2 -= $d;
        $y2 -= $d;
        $clipped = 1;
      }
      ($x1 <  $x_limit && $y1 <  $y_limit) or die;
      ($x2 <= $x_limit && $y2 <= $y_limit) or die;

      printwrap(" ($x1,$y1)--($x2,$y2)");
      $Printwrap = 8;
      last if $clipped;

      # U*[x1;y1]
      ($x1,$y1) = matrix_mult_xy(U_2x2(), $x1,$y1);
      ($x2,$y2) = matrix_mult_xy(U_2x2(), $x2,$y2);
    }
  }
  printwrap(";\n");
  print "    % end generated\n";

} else {
  die "unrecognised type: ",$type;
}

1;
